@@ -5,8 +5,10 @@
     created: Nov 4, 2017
     last update: February, 2019
 """
+import logging
 from logging import debug, info, warn, error
 import warnings
+logging.getLogger(__name__).addHandler(logging.NullHandler())
 
 class CircularBuffer(object):
     """
@@ -62,10 +64,12 @@ class CircularBuffer(object):
         >>> buffer = circular_buffer_numpy.circular_buffer.CircularBuffer(shape = (10,4)
         >>> from numpy.random import random
         >>> rand_arr = random(size=(6,4))
-        >>> buffer.apppend(rand_arr)
+        >>> buffer.append(rand_arr)
         >>> buffer.pointer
         5
         """
+        if len(data.shape) == len(self.shape)-1:
+            data = data.reshape((1,data.shape[0]))
         for i in range(data.shape[0]):
             if self.pointer == self.shape[0]-1:
                 self.pointer = -1
@@ -145,6 +149,27 @@ class CircularBuffer(object):
         """
         return self.get_last_N(N=self.shape[0])
 
+    def get_data(self):
+        """
+        return all valid circular buffer entries in ordered way, where
+        last value is the last collected.
+
+        Parameters
+        ----------
+
+        Returns
+        -------
+        array (numpy array)
+
+        Examples
+        --------
+        >>> data = circual_buffer.CircularBuffer.get_data()
+        """
+        if self.g_pointer + 1 < self.length:
+            return self.get_last_N(self.g_pointer+1)
+        else:
+            return self.get_all()
+
     def get_last_N(self, N):
         """
         returns last N entries from the known self.pointer(circular buffer pointer)
@@ -242,7 +267,7 @@ class CircularBuffer(object):
         if N-1 <= P:
             result = self.buffer[P+1-N:P+1]
         else:
-            result = concatenate((self.buffer[-(N-P-1):], self.buffer[:P+1]), axis=1)
+            result = concatenate((self.buffer[-(N-P-1):], self.buffer[:P+1]), axis=0)
         return result
 
     @property
@@ -266,6 +291,7 @@ class CircularBuffer(object):
         """
         integer: property objects that returns the size of the circular buffer.
         """
+        debug('returing size')
         return self.buffer.size
 
     @property
@@ -306,8 +332,11 @@ if __name__ == "__main__":  # for testing purposes
     from tempfile import gettempdir
     import traceback
 
+    #https://docs.python.org/3/library/logging.html#logrecord-attributes
+    from tempfile import gettempdir
     logging.basicConfig(filename=gettempdir()+'/circular_buffer_LL.log',
-                        level=logging.DEBUG, format="%(asctime)s %(levelname)s: %(message)s")
+                level=logging.DEBUG,
+                format="%(asctime)-15s|PID:%(process)-6s|%(levelname)-8s|%(name)s| module:%(module)s-%(funcName)s|message:%(message)s")
 
     print("Circular buffer library")
     print("two classes: server and client")
